package Net::DNS::Header;

use strict;
use warnings;

our $VERSION = (qw$Id: Header.pm 1910 2023-03-30 19:16:30Z willem $)[2];




use integer;
use Carp;

use Net::DNS::Parameters qw(:opcode :rcode);



sub string {
	my $self = shift;

	my $id	   = $self->id;
	my $qr	   = $self->qr;
	my $opcode = $self->opcode;
	my $rcode  = $self->rcode;
	my $qd	   = $self->qdcount;
	my $an	   = $self->ancount;
	my $ns	   = $self->nscount;
	my $ar	   = $self->arcount;
	return <<"QQ" if $opcode eq 'DSO';
;;	id = $id
;;	qr = $qr		opcode = $opcode	rcode = $rcode
;;	qdcount = $qd	ancount = $an	nscount = $ns	arcount = $ar
QQ
	return <<"QQ" if $opcode eq 'UPDATE';
;;	id = $id
;;	qr = $qr		opcode = $opcode	rcode = $rcode
;;	zocount = $qd	prcount = $an	upcount = $ns	adcount = $ar
QQ
	my $aa = $self->aa;
	my $tc = $self->tc;
	my $rd = $self->rd;
	my $ra = $self->ra;
	my $zz = $self->z;
	my $ad = $self->ad;
	my $cd = $self->cd;
	my $do = $self->do;
	return <<"QQ";
;;	id = $id
;;	qr = $qr	aa = $aa	tc = $tc	rd = $rd	opcode = $opcode
;;	ra = $ra	z  = $zz	ad = $ad	cd = $cd	rcode  = $rcode
;;	qdcount = $qd	ancount = $an	nscount = $ns	arcount = $ar
;;	do = $do
QQ
}



sub print {
	print &string;
	return;
}



my ( $cache1, $cache2, $limit );				# two layer cache

sub id {
	my ( $self, @value ) = @_;
	for (@value) { $$self->{id} = $_ }
	my $ident = $$self->{id};
	return $ident if $ident;
	return $ident if defined($ident) && $self->opcode eq 'DSO';
	( $cache1, $cache2, $limit ) = ( {0 => 1}, $cache1, 50 ) unless $limit--;
	$ident = int rand(0xffff);				# preserve short-term uniqueness
	$ident = int rand(0xffff) while $cache1->{$ident}++ + exists( $cache2->{$ident} );
	return $$self->{id} = $ident;
}



sub opcode {
	my ( $self, $arg ) = @_;
	my $opcode;
	for ( $$self->{status} ) {
		return opcodebyval( ( $_ >> 11 ) & 0x0f ) unless defined $arg;
		$opcode = opcodebyname($arg);
		$_	= ( $_ & 0x87ff ) | ( $opcode << 11 );
	}
	return $opcode;
}



sub rcode {
	my ( $self, $arg ) = @_;
	my $rcode;
	for ( $$self->{status} ) {
		my $opt = $$self->edns;
		unless ( defined $arg ) {
			$rcode = ( $opt->rcode & 0xff0 ) | ( $_ & 0x00f );
			$opt->rcode($rcode);			# write back full 12-bit rcode
			return $rcode == 16 ? 'BADVERS' : rcodebyval($rcode);
		}
		$rcode = rcodebyname($arg);
		$opt->rcode($rcode);				# full 12-bit rcode
		$_ &= 0xfff0;					# low 4-bit rcode
		$_ |= ( $rcode & 0x000f );
	}
	return $rcode;
}



sub qr {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x8000, @value );
}



sub aa {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0400, @value );
}



sub tc {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0200, @value );
}



sub rd {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0100, @value );
}



sub ra {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0080, @value );
}



sub z {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0040, @value );
}



sub ad {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0020, @value );
}



sub cd {
	my ( $self, @value ) = @_;
	return $self->_dnsflag( 0x0010, @value );
}



sub qdcount {
	my ( $self, @value ) = @_;
	for (@value) { $self->_warn('packet->header->qdcount is read-only') }
	return $$self->{count}[0] || scalar @{$$self->{question}};
}



sub ancount {
	my ( $self, @value ) = @_;
	for (@value) { $self->_warn('packet->header->ancount is read-only') }
	return $$self->{count}[1] || scalar @{$$self->{answer}};
}



sub nscount {
	my ( $self, @value ) = @_;
	for (@value) { $self->_warn('packet->header->nscount is read-only') }
	return $$self->{count}[2] || scalar @{$$self->{authority}};
}



sub arcount {
	my ( $self, @value ) = @_;
	for (@value) { $self->_warn('packet->header->arcount is read-only') }
	return $$self->{count}[3] || scalar @{$$self->{additional}};
}

sub zocount { return &qdcount; }
sub prcount { return &ancount; }
sub upcount { return &nscount; }
sub adcount { return &arcount; }



sub do {
	my ( $self, @value ) = @_;
	return $self->_ednsflag( 0x8000, @value );
}



sub size {				## historical
	my ( $self, @value ) = @_;
	return $$self->edns->UDPsize(@value);
}



sub edns {
	my $self = shift;
	return $$self->edns;
}



sub _dnsflag {
	my ( $self, $flag, @value ) = @_;
	for ( $$self->{status} ) {
		my $set = $_ | $flag;
		$_ = ( shift @value ) ? $set : ( $set ^ $flag ) if @value;
		$flag &= $_;
	}
	return $flag ? 1 : 0;
}


sub _ednsflag {
	my ( $self, $flag, @value ) = @_;
	my $edns = $$self->edns;
	for ( $edns->flags ) {
		my $set = $_ | $flag;
		$edns->flags( $_ = ( shift @value ) ? $set : ( $set ^ $flag ) ) if @value;
		$flag &= $_;
	}
	return $flag ? 1 : 0;
}


my %warned;

sub _warn {
	my ( undef, @note ) = @_;
	return carp "usage; @note" unless $warned{"@note"}++;
}


1;
__END__




